# -*- encoding: utf-8 -*-
"""
Utility for creating static html code.
"""

import argparse
import sys
import stat
import os.path
import shutil
import datetime

import envoy

try:  # pragma: no cover
    from ConfigParser import ConfigParser
except:  # pragma: no cover
    from configparser import ConfigParser

import logging
logging.basicConfig(level=logging.DEBUG)

from web_skeleton.getyesno import getyesno
from web_skeleton.getyesno import getstring

WEB_SKELETON_INI = 'web-skeleton.ini'

comandos = ['init', 'skeleton', 'render', 'gclass', 'rsync']


def yuno_js_code(yuno_name, yuno_role, root_name, root_gclass):
    code = """\
/**************************************************************************
 *        - Create Yuno
 *        - Setup routes
 *        - Create the root gobj:
 **************************************************************************/
(function(exports) {{
    'use strict';

    SMachine.set_machine_trace(true);   // global trace
    if(false) {{ // Set the logger in a browser window
        $('body').append('<div id="logger"></div>');

        SMachine.logger = function(msg) {{ // Set new logger
            $('#logger').append('<div>' + msg + '</div>');
        }};
    }}

    console.log("CREATING {yuno_role} yuno");

    var kw = {{
        trace_creation: true,
        trace_router: true
    }};
    var __yuno_gobj__ = new Yuno('', '{yuno_role}', kw);
    __yuno_gobj__.send_event(
        __yuno_gobj__.router,
        'EV_ADD_STATIC_ROUTE',
        {{
            name: '',
            role: 'yuneta_agent',
            urls: [
                'ws://127.0.0.1:5995'
            ],
        }}
    );

    $(function() {{
        // Start at onload
        console.log("======> Start RUNNING");
        $('#loading-message').remove();
        __yuno_gobj__.root = __yuno_gobj__.gobj_create(
           '{root_name}',
           {root_gclass},
           {{
               parent_dom_id:'body'
           }},
           __yuno_gobj__
        );
    }});


    /************************************************
     *          Expose to the global object
     ************************************************/
    exports.__yuno_gobj__ = __yuno_gobj__;

    if(typeof GLOBAL !== 'undefined') {{
        GLOBAL.__yuno_gobj__ = __yuno_gobj__;
    }}
}})(this);
""".format(
        yuno_name=yuno_name,
        yuno_role=yuno_role,
        root_name=root_name,
        root_gclass=root_gclass
    )
    return code


def get_gclass_code(name, page_dir=None):
    Name = name.capitalize()
    NAME = name.upper()
    code = """\
/**************************************************************************
 *      {Name} GClass
 **************************************************************************/
(function (exports) {{
    'use strict';

    /********************************************
     *      Auxiliary
     ********************************************/

    /********************************************
     *      Actions
     ********************************************/
    function ac_timeout(self, event, kw, src) {{
        console.log("ac_timeout");
        self.set_timeout(1*1000);
        return 0;
    }}

    var {NAME}_FSM = {{
        'event_list': [
            'EV_TIMEOUT'
        ],
        'state_list': [
            'ST_IDLE'
        ],
        'machine': {{
            'ST_IDLE':
            [
                ['EV_TIMEOUT',              ac_timeout,             undefined]
            ]
        }}
    }};

    var {NAME}_CONFIG = {{
        timeout_retry: 5,               // timeout retry, in seconds
        timeout_idle: 5,                // idle timeout, in seconds
        // Id of dom element parent. It has preference over parent gobj.
        parent_dom_id: ''
    }};

    var {Name} = GObj.__makeSubclass__();
    var proto = {Name}.prototype; // Easy access to the prototype
    proto.__init__= function(name, kw) {{
        this.name = name || '';  // set before super(), to put the same smachine name
        this.gclass_name = '{Name}';
        GObj.prototype.__init__.call(this, {NAME}_FSM, {NAME}_CONFIG);
        __update_dict__(this.config, kw || {{}});
        return this;
    }};

    /************************************************
     *      Framework Method create
     ************************************************/
    proto.mt_create = function() {{
        var self = this;
        self.set_timeout(1*1000);
    }}

    /************************************************
     *      Framework Method destroy
     *      In this point, all childs
     *      and subscriptions are already deleted.
     ************************************************/
    proto.mt_destroy= function() {{
    }}


    //=======================================================================
    //      Expose the class via the global object
    //=======================================================================
    exports.{Name} = {Name};

    if(typeof GLOBAL !== 'undefined') {{
        GLOBAL.{Name} = {Name};
    }}

}})(this);
""".format(
        Name=Name,
        NAME=NAME)
    return code


scss_page_code = """\
/*********************************************************
 *  Your scss code
 *********************************************************/
"""

setup_cfg_file = """\
[egg_info]
tag_build =
tag_svn_revision = false
tag_date = 0

[easy_install]
zip_ok = false
"""

setup_py_file = """\
# -*- encoding: utf-8 -*-

import os
import sys

from setuptools import setup, find_packages

if sys.version_info[:2] < (2, 6):
    raise RuntimeError('Requires Python 2.6 or better')

here = os.path.abspath(os.path.dirname(__file__))
try:
    README = open(os.path.join(here, 'README.rst')).read()
    CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()
except IOError:
    README = CHANGES = ''

version = '0.0.0'

requires = ['web_skeleton']

setup(name='{package}',
    version=version,
    description='{package}',
    long_description='',
    classifiers=[
        "Programming Language :: Python",
    ],
    author='',
    author_email='',
    url='',
    license='',
    keywords='web-skeleton yuneta',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=requires,
    tests_require=requires,
    test_suite="{package}.tests",
    entry_points='''\
    ''',
)
"""

watch_file_sh = """\
#!/bin/sh
make
watchfs -f watch.json
"""

watch_json_file = """\
{
    "global":{
        "Fs.info": false,
        "Watchfs.kw": {
            "info": false,
            "command":""rm -rf tags/0.00.aa/static/.sass-cache/ tags/0.00.aa/static/.webassets-cache/ tags/0.00.aa/.cache/; make"",
            "path":".",
            "recursive":true,
            "patterns":".*\\\\.py;.*\\\\.mako;.*\\\\.js;.*\\\\.css;.*\\\\.scss;.*\\\\.rst;.*\\\\.json;",
            "ignore_renamed_event":true
        }
    }
}
"""

makefile_file = """\
default: build

build:
\tweb-skeleton -vd render
"""

hgignore_file = """\
syntax:regexp

.directory
build/
dist/
data/
_build/
^\.tox/
\.svn/
^\.komodotools/
^\.settings/
\.cache/
\.egg-info/
\.webassets-cache/
\.sass-cache/
\.webassets-manifest

\.komodoproject$
\.coverage$
\.pyc$
\.so$
\.scss\.css$
\.egg$
^\.project$
^\.pydevproject$

\.tar\.gz$
"""

def main(argv=sys.argv):
    command = WebSkeleton(argv)
    return command.run()


class WebSkeleton(object):
    description = 'Generate static html code.'
    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument(
        "command",
        choices=comandos,
        help="Available commands (WARNING: to be executed in the web-skeleton.ini directory):\n"
"init {project} ==> create a .ini file and the directory structure.\n"
"skeleton ==> create a new tag directory, copying a assets directory.\n"
"render ==> generate a new index.html, in tags/{version} directory.\n"
"gclass {name} [sub-directory] ==> creat a new gclass (js/scss files) in main or path directory.\n"
"rsync ==> syncronize the tag version with the remote host.\n"
    )
    parser.add_argument(
        "arguments",
        nargs=argparse.REMAINDER,
        help="Arguments to command."
    )
    parser.add_argument(
        '-d',
        '--debug',
        dest='debug',
        action='store_true',
        help="True if render in development mode, False in production mode.",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        help="Increase output verbosity",
        action='store_true',
    )

    def __init__(self, argv=sys.argv):
        self.args = self.parser.parse_args(argv[1:])
        if self.args.debug:
            self.args.verbose = True

    def run(self):
        cmd = self.args.command
        fn = getattr(self, cmd)
        if cmd != 'init':
            self.load_ini()

        return fn()

    def init(self):
        """ create a .ini file and the directory structure
        """
        if not self.args.arguments:
            print('\nERROR: you must supply a project name!\n')
            return 2
        project = self.args.arguments[0].lower()
        output_dir = os.path.abspath(os.path.normpath(project))
        if os.path.exists(output_dir):
            print('\nERROR: directory "%s" already exists!\n' % (output_dir))
            return 2
        os.mkdir(output_dir)

        src_path = self.assets_dir()
        dst_path = os.path.join(output_dir, 'assets')
        shutil.copytree(src_path, dst_path, symlinks=True)

        src_path = self.code_dir()
        dst_path = os.path.join(output_dir, project, 'htmlrendercode')
        shutil.copytree(
            src_path,
            dst_path,
            ignore=shutil.ignore_patterns('*.pyc', '*~'),
            symlinks=True,
        )

        init_py = os.path.join(output_dir, project, '__init__.py')
        with open(init_py, 'w') as f:
            f.write("import htmlrendercode  "
                    "# needed to dynamic import from web_skeleton")

        setup_cfg = os.path.join(output_dir, 'setup.cfg')
        with open(setup_cfg, 'w') as f:
            f.write(setup_cfg_file)

        setup_py = os.path.join(output_dir, 'setup.py')
        with open(setup_py, 'w') as f:
            f.write(setup_py_file.format(package=project))

        # utilities
        watch = os.path.join(output_dir, 'watch.sh')
        with open(watch, 'w') as f:
            f.write(watch_file_sh)
        st = os.stat(watch)
        os.chmod(watch, st.st_mode | stat.S_IEXEC)

        watch = os.path.join(output_dir, 'watch.json')
        with open(watch, 'w') as f:
            f.write(watch_json_file)

        makefile = os.path.join(output_dir, 'Makefile')
        with open(makefile, 'w') as f:
            f.write(makefile_file)

        hgignore = os.path.join(output_dir, '.hgignore')
        with open(hgignore, 'w') as f:
            f.write(hgignore_file)

        os.mkdir(os.path.join(output_dir, 'tags'))
        current_tag = '0.00.aa'
        config = ConfigParser()
        config.set('DEFAULT', 'current_tag', current_tag)
        config.add_section('tags')
        config.set('tags', current_tag, '')
        config.add_section(current_tag)
        config.set(
            current_tag,
            'assets',
            os.path.join('assets', 'h5bp'),
        )
        config.set(
            current_tag,
            'remote-server',
            '',
        )

        ini_file = os.path.join(output_dir, WEB_SKELETON_INI)
        with open(ini_file, 'w') as configfile:
            config.write(configfile)

        print('\nOK: created new web-skeleton project in "%s".\n' % output_dir)
        print('Next steps:')
        print('   - Go to %s directory (cd %s)' % (project, project))
        print('   - Change definitions in base.py (page_data)')
        print('   - Check your settings in web-skeleton.ini file')
        print('   - Build a new h5bp structure (web-skeleton skeleton)')
        print('')

        return 0

    def load_ini(self):
        if not os.path.exists(WEB_SKELETON_INI):
            print('\nERROR: file "%s" NOT found.\n' % (WEB_SKELETON_INI))
            exit(2)
        # 'here' is the directory of the .ini file
        here = os.path.dirname(os.path.abspath(WEB_SKELETON_INI))
        self.config = ConfigParser({'here': here})
        self.config.read(WEB_SKELETON_INI)
        self.config.here = here
        self.config.yuno_name = ''
        self.config.yuno_role = ''  # os.path.basename(os.getcwd())
        self.config.root_name = ''
        self.config.root_gclass = ''
        self.config.current_tag = self.config.get('DEFAULT', 'current_tag')

    def module_dir(self):
        mod = sys.modules[self.__class__.__module__]
        return os.path.dirname(mod.__file__)

    def assets_dir(self):
        return os.path.join(self.module_dir(), "assets")

    def code_dir(self):
        return os.path.join(self.module_dir(), "_htmlrendercode")

    def current_tag_dir(self):
        return os.path.join(
            self.config.here,
            'tags',
            self.config.current_tag,
        )

    def skeleton(self):
        """ create a new tag directory, copying a assets directory
        """
        dst_path = self.current_tag_dir()

        if not os.path.exists(dst_path):
            pass # print('<---- * Creating "%s" directory.' % (dst_path))
        else:
            resp = getyesno(
                'WARNING: You are re-creating the skeleton "%s".\n'
                'You will loose your data! Are you sure?' % dst_path,
                default='n',
            )
            if not resp:
                print('\nOperation aborted.\n')
                return 2
            shutil.rmtree(dst_path)

        self.config.yuno_role = ''
        while not self.config.yuno_role:
            self.config.yuno_role = getstring(
                'Enter yuno role (maximum 15 characters):',
                ''
            )

        self.config.yuno_name = getstring(
            "Enter yuno name ['']:",
            ''
        )
        self.config.root_name = getstring(
            "Enter root gobj name ['root']:",
            'root'
        )
        self.config.root_gclass = self.config.root_name.capitalize()
        self.config.author = getstring(
            "Enter author ['']:",
            ''
        )
        self.config.license = getstring(
            "Enter license ['MIT']:",
            'MIT'
        )
        self.config.version = getstring(
            "Enter version ['1.0.0']:",
            '1.0.0'
        )

        assets = self.config.get(self.config.current_tag, 'assets')
        src_path = os.path.join(
            self.config.here,
            assets,
        )
        shutil.copytree(
            src_path,
            dst_path,
            symlinks=True,
        )

        # creat links of default pages
        self.gclass(
            'base',
            None,
            yuno_js_code(
                self.config.yuno_name,
                self.config.yuno_role,
                self.config.root_name,
                self.config.root_gclass
            )
        )
        print('\nOK: Created "%s" tag.\n' % (dst_path))

        print('Next steps:')
        print('   - Create your root gclass (web-skeleton gclass {name} [sub-directory]')
        print('   - Update the assets.py file with new js file')
        print('   - Render (web-skeleton render)')
        print('')

        return 0

    def gclass(self, name=None, page_dir=None, gclass_code=None):
        """ creat a new gclass, set of js/scss files
        """
        if not name:
            #print('====> page')
            if not self.args.arguments:
                print('WARNING: You must supply a name!')
                return 2
            name = self.args.arguments[0].lower()
            if len(self.args.arguments) >= 2:
                page_dir = self.args.arguments[1]
        if not gclass_code:
            gclass_code = get_gclass_code(name)
        self._make_set(
            name,
            gclass_code,
            scss_page_code,
            page_dir
        )

    def _make_set(
            self,
            name,
            gclass_code,
            scss_page_code,
            page_dir=None):
        project = os.path.basename(self.config.here).lower()

        if page_dir:
            rendercode_path = os.path.join(
                self.config.here,
                project,
                'htmlrendercode',
                page_dir
            )
        else:
            rendercode_path = os.path.join(
                self.config.here,
                project,
                'htmlrendercode'
            )

        if not os.path.exists(self.current_tag_dir()):
            print('\nWARNING: firstly you must create a tag skeleton.\n')
            exit(2)

        if not os.path.exists(rendercode_path):
            os.mkdir(rendercode_path)

        js_file = '%s.js' % name
        scss_file = '%s.scss' % name

        ## Javascript
        filename = os.path.join(rendercode_path, js_file)
        if not os.path.exists(filename):
            print('Creating "%s" file...' % (filename))
            Name = name.capitalize()
            fd = open(filename, 'w')
            fd.write(gclass_code)
            fd.close()

        ## Scss
        filename = os.path.join(rendercode_path, scss_file)
        if not os.path.exists(filename):
            print('Creating "%s" file...' % (filename))
            fd = open(filename, 'w')
            fd.write(scss_page_code)
            fd.close()

        ## scss symbolic link
        if not page_dir:
            ln_path = os.path.join(
                self.current_tag_dir(),
                'static',
                'css',
                'app',
            )
        else:
            ln_path = os.path.join(
                self.current_tag_dir(),
                'static',
                'css',
                page_dir,
            )
        if not os.path.exists(ln_path):
            os.mkdir(ln_path)

        os.chdir(ln_path)
        if not page_dir:
            source_file = '../../../../../%s/htmlrendercode/%s' % (
                project, scss_file)
        else:
            source_file = '../../../../../%s/htmlrendercode/%s/%s' % (
                project,
                page_dir,
                scss_file,
            )
        link_name = scss_file
        try:
            os.symlink(source_file, link_name)
            print('Creating "%s" symbolic link...' % (
                ln_path + '/' + link_name))
            msg = """Remember to add to scss_content[] list (assets.py) \
the line:
    'css/%s/%s.scss'\n""" % (
                'app' if not page_dir else page_dir,
                name
            )
            print(msg)
        except OSError:
            pass

        ## js symbolic link
        if not page_dir:
            ln_path = os.path.join(
                self.current_tag_dir(),
                'static',
                'js',
                'bottom',
                'app',
            )
        else:
            ln_path = os.path.join(
                self.current_tag_dir(),
                'static',
                'js',
                'bottom',
                page_dir,
            )
        if not os.path.exists(ln_path):
            os.mkdir(ln_path)

        os.chdir(ln_path)
        if not page_dir:
            source_file = '../../../../../../%s/htmlrendercode/%s' % (
                project, js_file)
        else:
            source_file = '../../../../../../%s/htmlrendercode/%s/%s' % (
                project,
                page_dir,
                js_file
            )
        link_name = js_file
        try:
            os.symlink(source_file, link_name)
            print('Creating "%s" symbolic link...' % (
                ln_path + '/' + link_name))
            msg = """Remember to add to bottom_js_content[] list (assets.py) \
the line:
    'js/bottom/%s/%s.js'\n""" % (
                'app' if not page_dir else page_dir,
                name,
            )
            print(msg)
        except OSError:
            pass


        return 0

    def render(self):
        """ generate a new index.html, in tags/{version} directory
        WebSkeleton will render using next call code:

        get_base(here, output_path, debug)
            is the only function being called from web_skeleton.
            The rest is up to you.
            This function must return a class with a `render` method.
            The `render` method must return a string with the html code.
            :param here: path where project web-skeleton resides.
            :param output_path: current output tag directory.
            :param debug: True if you want debug.
        """
        project = os.path.basename(self.config.here).lower()
        if self.args.verbose:
            print('Rendering ' + project + '...')
        output_path = self.current_tag_dir()
        if not os.path.exists(output_path):
            print('\nWARNING: firstly you must create a tag skeleton.\n')
            exit(2)

        here = os.path.join(
            self.config.here,
            #'htmlrendercode',
        )
        sys.path.append(here)
        os.chdir(self.config.here)

        package = __import__(project, globals(), locals(), [], -1)
        if not package:
            print("ERROR: package '%s' NOT FOUND" % project)
            return
        if not hasattr(package, "htmlrendercode"):
            print("ERROR: package '%s' has NOT htmlrendercode module" % project)
            return
        try:
            base = package.htmlrendercode.base.get_base(
                here,
                output_path,
                self.args.debug
            )
        except AttributeError:
            # compatible with versions 0.3.*
            #base = package.htmlrendercode.page.get_page(
            #    here,
            #    output_path,
            #    self.args.debug
            #)
            raise

        html = base.render()
        if self.args.verbose:
            print(html)

        index_html = os.path.join(output_path, 'index.html')
        fd = open(index_html, 'w')
        fd.write(html)
        fd.close()
        print('\nOK: Created "%s" file.\n' % (index_html,))
        return 0

    def rsync(self):
        """ syncronize the tag version with the remote host
        """
        src_path = os.path.join(self.current_tag_dir(), '')
        remote = self.config.get(self.config.current_tag, 'remote-server')
        if not remote:
            print('\nWARNING: Please specify a remote server path.\n')
            exit(2)
        print('rsyncing...')

        command = r'rsync -avzL --delete ' \
            r'--exclude \.webassets-cache --exclude \.sass-cache --exclude \.cache %s %s' % (
                src_path,
                remote,
            )
        if self.args.verbose:
            print(command)
        response = envoy.run(command)
        print(response.std_out)

        return 0


if __name__ == '__main__':
    main()
